/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.adapter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Collection;

import middlegen.Column;
import middlegen.ColumnMap;
import middlegen.RelationshipRole;
import middlegen.Table;
import middlegen.Util;
import middlegen.DbNameConverter;
import middlegen.javax.JavaTable;

/**
 * A decorated table used by the Adapter Plugin. It has some additional methods
 * needed to generate Adapter code.
 *
 * @author <a href="mailto:tfogwill@csir.co.za">Thomas Fogwill</a>
 * @created May 12, 2003
 */
public class AdapterTable extends JavaTable {

   /**
    * Constructs a neww AdapterTable by decorating the provided table.
    *
    * @param table The table to be decorated
    */
   public AdapterTable(Table table) {
      super(table);
   }


   /**
    * Gets the Java name attribute of this table (i.e. the name given to objects
    * representing records in this table).
    *
    * @return The Java name as a String
    */
   public String getJavaName() {
      return getBaseClassName();
   }


   /**
    * Gets the foreign keys in this table
    *
    * @return The foreign keys, in a list of middlegen.plugins.adapter.AdapterFK
    *      objects
    */
   public Collection getForeignKeys() {
      List keys = new ArrayList();
      for (Iterator iter = getRelationshipRoles().iterator(); iter.hasNext(); ) {
         RelationshipRole rel = (RelationshipRole)iter.next();
         if (rel.isTargetPrimaryKey()) {
            System.out.println(" -- " + rel.getName());
            ColumnMap colMap = rel.getTargetRole().getColumnMaps()[0];
            String localTable = getBaseClassName();
            String localColumn = colMap.getForeignKey();
            String foreignTableSqlName = rel.getTarget(getPlugin())
                  .getSqlName();
            String foreignTableBaseClassName =
                  DbNameConverter.getInstance()
                  .tableNameToVariableName(rel.getTarget(getPlugin())
                  .getSqlName());
            foreignTableBaseClassName =
                  Util.singularise(foreignTableBaseClassName);
            String foreignColumn = colMap.getPrimaryKey();
            keys.add(new AdapterFK(localTable, localColumn,
                  foreignTableSqlName,
                  foreignTableBaseClassName,
                  foreignColumn, rel));
         }
      }
      return keys;
   }
}
