/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import middlegen.util.BooleanNode;
import middlegen.swing.JColumnSettingsPanel;
import middlegen.swing.JTableSettingsPanel;
import middlegen.plugins.entitybean.swing.JEntity20ColumnSettingsPanel;
import middlegen.plugins.entitybean.swing.JEntityTableSettingsPanel;
import org.apache.tools.ant.types.EnumeratedAttribute;

/**
 * This plugin generates Enterprise Java Beans of type EntityBean conforming to
 * the EJB 2.0 specification.
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 22. mars 2002
 */
public abstract class Entity20Plugin extends Entity11Plugin {

   /**
    * @todo-javadoc Describe the column
    */
   private String _localsuffix = "Local";
   /**
    * @todo-javadoc Describe the column
    */
   private String _localhomesuffix = "LocalHome";

   /**
    * @todo-javadoc Describe the column
    */
   private final BooleanNode _isLocal = BooleanNode.createRoot(false);

   /**
    * The view type. defaults to local
    */
   private String _viewtype = ViewType.LOCAL;

   /**
    * Get static reference to Log4J Logger
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Entity20Plugin.class.getName());


   /**
    * Describe what the Entity20Plugin constructor does
    *
    * @todo-javadoc Write javadocs for method parameter
    * @todo-javadoc Write javadocs for constructor
    * @todo-javadoc Write javadocs for method parameter
    */
   protected Entity20Plugin() {
      super();
   }



   /**
    * Sets the Localsuffix attribute of the EntityClassType object
    *
    * @param s The new Localsuffix value
    */

   public void setLocalsuffix(String s) {
      _localsuffix = s;
   }


   /**
    * Sets the Localhomesuffix attribute of the EntityClassType object
    *
    * @param s The new Localhomesuffix value
    */
   public void setLocalhomesuffix(String s) {
      _localhomesuffix = s;
   }


   /**
    * Sets the Viewtype attribute of the CMP20Plugin object
    *
    * @param vt The new Viewtype value
    */
   public void setViewtype(ViewType vt) {
      _viewtype = vt.getValue();
   }



   /**
    * Gets the Viewtype attribute of the Entity20Plugin object
    *
    * @return The Viewtype value
    */
   public String getViewtype() {
      return _viewtype;
   }


   /**
    * Gets the ColumnSettingsPanel attribute of the EntityCMP20ClassType object
    *
    * @return The ColumnSettingsPanel value
    */
   public JColumnSettingsPanel getColumnSettingsPanel() {
      return new JEntity20ColumnSettingsPanel();
   }


   /**
    * Gets the TableSettingsPanel attribute of the EntityCMP20ClassType object
    *
    * @return The TableSettingsPanel value
    */
   public JTableSettingsPanel getTableSettingsPanel() {
      return new JEntityTableSettingsPanel();
   }


   /**
    * Gets the Localsuffix attribute of the Entity20Plugin object
    *
    * @return The Localsuffix value
    */
   public String getLocalsuffix() {
      return _localsuffix;
   }


   /**
    * Gets the Localhomesuffix attribute of the Entity20Plugin object
    *
    * @return The Localhomesuffix value
    */
   public String getLocalhomesuffix() {
      return _localhomesuffix;
   }


   /**
    * Gets the ColumnDecoratorClass attribute of the Entity20Plugin object
    *
    * @return The ColumnDecoratorClass value
    */
   public Class getColumnDecoratorClass() {
      return Entity20Column.class;
   }


   /**
    * Gets the TableDecoratorClass attribute of the Entity20Plugin object
    *
    * @return The TableDecoratorClass value
    */
   public Class getTableDecoratorClass() {
      return Entity20Table.class;
   }


   /**
    * Gets the Local attribute of the Entity20Plugin object
    *
    * @return The Local value
    */
   public BooleanNode getLocal() {
      return _isLocal;
   }


   /**
    * Describe what this class does
    *
    * @author Aslak Hellesy
    * @created 30. juni 2002
    * @todo-javadoc Write javadocs
    */
   public static class ViewType extends EnumeratedAttribute {
      /**
       * @todo-javadoc Describe the field
       */
      public final static String LOCAL = "local";
      /**
       * @todo-javadoc Describe the field
       */
      public final static String BOTH = "both";
      /**
       * @todo-javadoc Describe the field
       */
      public final static String REMOTE = "remote";


      /**
       * Gets the Values attribute of the StrutsVersion object
       *
       * @return The Values value
       */
      public java.lang.String[] getValues() {
         return (new java.lang.String[]{
               LOCAL, BOTH, REMOTE
               });
      }
   }
}
