/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen.plugins.entitybean;

import java.util.Iterator;

import middlegen.Table;
import middlegen.Column;
import middlegen.predicates.column.PrimaryKey;
import middlegen.MiddlegenException;

/**
 * This is the super class for the server specific sub tasks for the entity bean
 * task.
 *
 * @author Eivind Waaler
 * @author Aslak Hellesy
 * @created 4. juli 2002
 * @todo-javadoc Write javadocs
 */
public abstract class Server {

   /**
    * @todo-javadoc Describe the field
    */
   private Entity11Plugin _plugin;

   /**
    * @todo-javadoc Describe the field
    */
   private static org.apache.log4j.Category _log = org.apache.log4j.Category.getInstance(Server.class.getName());


   /**
    * Gets the ServerName attribute of the Server object
    *
    * @return The ServerName value
    */
   public abstract String getServerName();


   /**
    * Performs server-specific validations.
    *
    * @exception MiddlegenException Describe the exception
    * @todo-javadoc Write javadocs for exception
    */
   public void validate() throws MiddlegenException {
      warnIfPkWithFkExistsAndNotFkCmp();
   }


   /**
    * Gets the Plugin attribute of the Server object
    *
    * @return The Plugin value
    */
   protected Entity11Plugin getPlugin() {
      return _plugin;
   }


   /**
    * Sets the Plugin attribute of the Server object
    *
    * @param plugin The new Plugin value
    */
   void setPlugin(Entity11Plugin plugin) {
      _plugin = plugin;
   }


   /**
    * Logs a warning if there are any primary keys where one or more of the
    * columns are also foreign keys AND fkcmp="false". <p>
    *
    * This is because the key won't be correctly persisted if this is true.
    */
   private void warnIfPkWithFkExistsAndNotFkCmp() {
      if (getPlugin() instanceof CMP20Plugin) {
         CMP20Plugin cmp20Plugin = (CMP20Plugin)getPlugin();
         if (!cmp20Plugin.isFkcmp()) {
            for (Iterator tables = getPlugin().getMiddlegen().getTables().iterator(); tables.hasNext(); ) {
               Table table = (Table)tables.next();
               boolean comma = false;
               StringBuffer fks = new StringBuffer();
               for (Iterator columns = table.getColumns(PrimaryKey.getInstance()).iterator(); columns.hasNext(); ) {
                  Column column = (Column)columns.next();
                  if (column.isFk()) {
                     // Yuk!
                     if (comma) {
                        fks.append(',');
                     }
                     fks.append(column.getSqlName());
                     comma = true;
                  }
               }
               if (fks.length() > 0) {
                  _log.warn("WARNING (" + getPlugin().getName() + "): The table " + table.getSqlName() + " has a primary key where " +
                        "the following columns also are foreign keys: " + fks.toString() +
                        ". Having set fkcmp=\"false\" this will most likely cause problems at run time " +
                        "because those columns will not be persisted, and therefore new instances will not " +
                        "set the key fields that correspond to foreign keys. The database might choke on this."
                        );
               }
            }
         }
      }
   }
}
