package ejb.persons.entity;

/**
 * This class is part of Middlegen airlines, and it is
 * a CMP EJB accessing the persons table.
 *
 * @author <a href="http://boss.bekk.no/boss/middlegen/">Middlegen</a>
 *
 * @ejb:finder
 *    signature="java.util.Collection findByFirstNameAndLastName(java.lang.String firstName,java.lang.String lastName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.firstName = ?1 AND o.lastName = ?2"
 *
 * @ejb.bean
 *    type="CMP"
 *    cmp-version="2.x"
 *    name="Person"
 *    local-jndi-name="airline/PersonLocalHome"
 *    view-type="local"
 *    primkey-field="personId"
 *
 * @ejb.value-object
 *    name="Person"
 *    extends="ejb.persons.entity.PersonLightValue"
 *	  match="*"
 *	  instantiation="eager"
 *
 * @ejb.value-object
 *    name="PersonLight"
 *	  match="light"
 *
 *	@ejb.facade 
 *    type="Stateless"
 *    view-type="remote"
 *
 *
 * @ejb.finder
 *    signature="java.util.Collection findAll()"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT OBJECT(o) FROM Person o"
 *
 * @ejb.finder
 *    signature="java.util.Collection findByFirstName(java.lang.String firstName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.firstName = ?1"
 *    description="first_name is not indexed."
 * @ejb.finder
 *    signature="java.util.Collection findByLastName(java.lang.String lastName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.lastName = ?1"
 *    description="last_name is not indexed."
 * @ejb.persistence table-name="persons"
 * @ejb.transaction type="Required" 
 *
 * @weblogic.data-source-name jdbc_1
 *
 * @jonas.bean
 *      ejb-name="Person"
 *      jndi-name="airline/PersonLocalHome"
 *      cleanup="none"
 *
 * @jonas.jdbc-mapping
 *       jndi-name="jdbc_1"
 *       jdbc-table-name="persons"	
 *       automatic_pk="true"
 *
 */

public abstract class PersonBean implements javax.ejb.EntityBean {

   /**
    * Returns the personId
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the personId
    *
    * @ejb.pk-field
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="person_id"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="personId"
     *	jdbc-field-name="person_id"
    */
   public abstract java.lang.Integer getPersonId();

   /**
    * Sets the personId
    *
    * @param personId the new personId value
    */
   public abstract void setPersonId(java.lang.Integer personId);

   /**
    * Returns the firstName
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the firstName
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="first_name"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="firstName"
     *	jdbc-field-name="first_name"
    */
   public abstract java.lang.String getFirstName();

   /**
    * Sets the firstName
    *
    * @param firstName the new firstName value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setFirstName(java.lang.String firstName);

   /**
    * Returns the lastName
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the lastName
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="last_name"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="lastName"
     *	jdbc-field-name="last_name"
    */
   public abstract java.lang.String getLastName();

   /**
    * Sets the lastName
    *
    * @param lastName the new lastName value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setLastName(java.lang.String lastName);

   /**
    * This is a bi-directional one-to-many relationship CMR method
    *
    * @return a java.util.Collection of related ejb.reservations.entity.ReservationLocal.
    *
    * @ejb.interface-method view-type="local"
    *
    * @ejb.relation
    *    name="persons-cmp20-reservations-cmp20"
    *    role-name="persons-cmp20-has-reservations-cmp20"
    *
    * @jboss.relation-mapping style="foreign-key"
    *
    * @ejb.value-object
    *   aggregate="ejb.reservations.entity.ReservationLightValue"
    *   aggregate-name="reservations"
    *   members="ejb.reservations.entity.ReservationLocal"
    *   members-name="reservations"
    *   match="all"
    *   relation="external"
    *   type="java.util.Collection"
    */
   public abstract java.util.Collection getReservations();

   /**
    * Sets a collection of related ejb.reservations.entity.ReservationLocal
    *
    * @param a collection of related ejb.reservations.entity.ReservationLocal
    *
    * @ejb.interface-method view-type="local"
    *
    * @param reservations the new CMR value
    */
   public abstract void setReservations(java.util.Collection reservations);

   /**
    * This create method takes only mandatory (non-nullable) parameters.
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param personId the personId value
    * @param firstName the firstName value
    * @param lastName the lastName value
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public java.lang.Integer ejbCreate( java.lang.Integer personId, java.lang.String firstName, java.lang.String lastName ) throws javax.ejb.CreateException {
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      // Set CMP fields
      setPersonId(personId);
      setFirstName(firstName);
      setLastName(lastName);
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes thos method immediately after it calls ejbCreate. 
    *
    * @param personId the personId value
    * @param firstName the firstName value
    * @param lastName the lastName value
    */
   public void ejbPostCreate( java.lang.Integer personId, java.lang.String firstName, java.lang.String lastName ) throws javax.ejb.CreateException {
      // Set CMR fields
   }

  
   /**
    * Return the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract ejb.persons.entity.PersonLightValue getPersonLightValue();

   /**
    * Set the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setPersonLightValue(ejb.persons.entity.PersonLightValue value);

   /**
    * Return the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract ejb.persons.entity.PersonValue getPersonValue();
   
   /**
    * Set the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setPersonValue(ejb.persons.entity.PersonValue value);

   /**
    * Create and return a value object populated with the data from
    * this bean.
    *
    * Standard method that must be on all Beans for the TreeBuilder to
    * work its magic.
    *
    * @return Returns a value object containing the data within this bean.
    *
    * @ejb.interface-method view-type="local"
    */
   public ejb.persons.entity.PersonValue getValueObject() {
      ejb.persons.entity.PersonValue valueObject = new ejb.persons.entity.PersonValue();
      valueObject.setPersonId(getPersonId());
      valueObject.setFirstName(getFirstName());
      valueObject.setLastName(getLastName());
      return valueObject;
   }

   /**
    * Creates an instance based on a value object
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param value the value object used to initialise the new instance
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public java.lang.Integer ejbCreate(ejb.persons.entity.PersonLightValue value) throws javax.ejb.CreateException {
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      setPersonId(value.getPersonId());
      setFirstName(value.getFirstName());
      setLastName(value.getLastName());
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes this method immediately after it calls ejbCreate. 
    *
    * @param value the value object used to initialise the new instance
    */
   public void ejbPostCreate(ejb.persons.entity.PersonLightValue value) throws javax.ejb.CreateException {
   }
   // No H:\Projet\Middlegen\samples/src/middlegen/cmp20-persons-class-code.txt found.

  public void ejbLoad() 
   {
   }

   public void ejbStore() 
   {
   }

   public void ejbActivate() 
   {
   }

   public void ejbPassivate() 
   {
   }

   public void setEntityContext(javax.ejb.EntityContext ctx) 
   {
   }

   public void unsetEntityContext() 
   {
   }

   public void ejbRemove() throws javax.ejb.RemoveException
   {

   }


}
