package ejb.reservations.entity;

/**
 * This class is part of Middlegen airlines, and it is
 * a CMP EJB accessing the reservations table.
 *
 * @author <a href="http://boss.bekk.no/boss/middlegen/">Middlegen</a>
 *
 *
 * @ejb.bean
 *    type="CMP"
 *    cmp-version="2.x"
 *    name="Reservation"
 *    local-jndi-name="airline/ReservationLocalHome"
 *    view-type="local"
 *
 * @ejb.value-object
 *    name="Reservation"
 *    extends="ejb.reservations.entity.ReservationLightValue"
 *	  match="*"
 *	  instantiation="eager"
 *
 * @ejb.value-object
 *    name="ReservationLight"
 *	  match="light"
 *
 *	@ejb.facade 
 *    type="Stateless"
 *    view-type="remote"
 *
 *
 * @ejb.finder
 *    signature="java.util.Collection findAll()"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT OBJECT(o) FROM Reservation o"
 *
 * @ejb.finder
 *    signature="java.util.Collection findByRegistrationUtc(java.sql.Timestamp registrationUtc)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Reservation o WHERE o.registrationUtc = ?1"
 *    description="registration_utc is indexed."
 * @ejb.finder
 *    signature="java.util.Collection findByComment(java.lang.String comment)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Reservation o WHERE o.comment = ?1"
 *    description="comment is not indexed."
 * @ejb.persistence table-name="reservations"
 * @ejb.transaction type="Required" 
 *
 * @weblogic.data-source-name jdbc_1
 *
 * @jonas.bean
 *      ejb-name="Reservation"
 *      jndi-name="airline/ReservationLocalHome"
 *      cleanup="none"
 *
 * @jonas.jdbc-mapping
 *       jndi-name="jdbc_1"
 *       jdbc-table-name="reservations"	
 *
 */

public abstract class ReservationBean implements javax.ejb.EntityBean {

   /**
    * Returns the reservationId
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the reservationId
    *
    * @ejb.pk-field
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="reservation_id"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="reservationId"
     *	jdbc-field-name="reservation_id"
    */
   public abstract java.lang.Integer getReservationId();

   /**
    * Sets the reservationId
    *
    * @param reservationId the new reservationId value
    */
   public abstract void setReservationId(java.lang.Integer reservationId);

   /**
    * Returns the registrationUtc
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the registrationUtc
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="registration_utc"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="registrationUtc"
     *	jdbc-field-name="registration_utc"
    */
   public abstract java.sql.Timestamp getRegistrationUtc();

   /**
    * Sets the registrationUtc
    *
    * @param registrationUtc the new registrationUtc value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setRegistrationUtc(java.sql.Timestamp registrationUtc);

   /**
    * Returns the comment
    * @todo support OracleClob,OracleBlob on WLS
    *
    * @return the comment
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="comment"
    * @ejb.value-object match="light"
     * @jonas.cmp-field-jdbc-mapping
     *	field-name="comment"
     *	jdbc-field-name="comment"
    */
   public abstract java.lang.String getComment();

   /**
    * Sets the comment
    *
    * @param comment the new comment value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setComment(java.lang.String comment);

   /**
    * This is a bi-directional one-to-many relationship CMR method
    *
    * @return the related ejb.flights.entity.FlightLocal.
    *
    * @ejb.interface-method view-type="local"
    *
    * @ejb.relation
    *    name="flights-cmp20-reservations-cmp20"
    *    role-name="reservations-cmp20-has-flights-cmp20"
    *
    * @jboss.relation-mapping style="foreign-key"
    *
    * @weblogic.column-map
    *    foreign-key-column="flight_id_fk"
    *    key-column="flight_id"
    *
    * @jboss.relation
    *    fk-constraint="true"
    *    fk-column="flight_id_fk"
    *    related-pk-field="flightId"
    *
    * @jonas.ejb-relation
    *       ejb-relation-name="flights-cmp20-reservations-cmp20"
    *       ejb-relationship-role-name1="reservations-cmp20-has-flights-cmp20"
    *       foreign-key-jdbc-name1="flight_id_fk"
    *	    key-jdbc-name1="flight_id"
    * @ejb.value-object
    *   aggregate="ejb.flights.entity.FlightLightValue"
    *   aggregate-name="flight"
    *   match="all"
    *   relation="external"
    */
   public abstract ejb.flights.entity.FlightLocal getFlight();

   /**
    * Sets the related ejb.flights.entity.FlightLocal
    *
    * @param ejb.reservations.entity.ReservationLocal the related $target.variableName
    *
    * @ejb.interface-method view-type="local"
    *
    * @param flight the new CMR value
    */
   public abstract void setFlight(ejb.flights.entity.FlightLocal flight);

   /**
    * This is a bi-directional one-to-many relationship CMR method
    *
    * @return the related ejb.persons.entity.PersonLocal.
    *
    * @ejb.interface-method view-type="local"
    *
    * @ejb.relation
    *    name="persons-cmp20-reservations-cmp20"
    *    role-name="reservations-cmp20-has-persons-cmp20"
    *
    * @jboss.relation-mapping style="foreign-key"
    *
    * @weblogic.column-map
    *    foreign-key-column="person_id_fk"
    *    key-column="person_id"
    *
    * @jboss.relation
    *    fk-constraint="true"
    *    fk-column="person_id_fk"
    *    related-pk-field="personId"
    *
    * @jonas.ejb-relation
    *       ejb-relation-name="persons-cmp20-reservations-cmp20"
    *       ejb-relationship-role-name1="reservations-cmp20-has-persons-cmp20"
    *       foreign-key-jdbc-name1="person_id_fk"
    *	    key-jdbc-name1="person_id"
    * @ejb.value-object
    *   aggregate="ejb.persons.entity.PersonLightValue"
    *   aggregate-name="person"
    *   match="all"
    *   relation="external"
    */
   public abstract ejb.persons.entity.PersonLocal getPerson();

   /**
    * Sets the related ejb.persons.entity.PersonLocal
    *
    * @param ejb.reservations.entity.ReservationLocal the related $target.variableName
    *
    * @ejb.interface-method view-type="local"
    *
    * @param person the new CMR value
    */
   public abstract void setPerson(ejb.persons.entity.PersonLocal person);

   /**
    * This create method takes only mandatory (non-nullable) parameters.
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param reservationId the reservationId value
    * @param registrationUtc the registrationUtc value
    * @param flight mandatory CMR field
    * @param person mandatory CMR field
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public ejb.reservations.entity.ReservationPK ejbCreate( java.lang.Integer reservationId, java.sql.Timestamp registrationUtc, ejb.flights.entity.FlightLocal flight, ejb.persons.entity.PersonLocal person ) throws javax.ejb.CreateException {
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      // Set CMP fields
      setReservationId(reservationId);
      setRegistrationUtc(registrationUtc);
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes thos method immediately after it calls ejbCreate. 
    *
    * @param reservationId the reservationId value
    * @param registrationUtc the registrationUtc value
    * @param flight mandatory CMR field
    * @param person mandatory CMR field
    */
   public void ejbPostCreate( java.lang.Integer reservationId, java.sql.Timestamp registrationUtc, ejb.flights.entity.FlightLocal flight, ejb.persons.entity.PersonLocal person ) throws javax.ejb.CreateException {
      // Set CMR fields
      setFlight(flight);
      setPerson(person);
   }

   /**
    * This create method takes all parameters (both nullable and not nullable). 
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param reservationId the reservationId value
    * @param registrationUtc the registrationUtc value
    * @param comment the comment value
    * @param flight CMR field
    * @param person CMR field
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public ejb.reservations.entity.ReservationPK ejbCreate( java.lang.Integer reservationId, java.sql.Timestamp registrationUtc, java.lang.String comment, ejb.flights.entity.FlightLocal flight, ejb.persons.entity.PersonLocal person ) throws javax.ejb.CreateException {
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      // Set CMP fields
      setReservationId(reservationId);
      setRegistrationUtc(registrationUtc);
      setComment(comment);
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes this method immediately after it calls ejbCreate. 
    *
    * @param reservationId the reservationId value
    * @param registrationUtc the registrationUtc value
    * @param comment the comment value
    * @param flight CMR field
    * @param person CMR field
    */
   public void ejbPostCreate( java.lang.Integer reservationId, java.sql.Timestamp registrationUtc, java.lang.String comment, ejb.flights.entity.FlightLocal flight, ejb.persons.entity.PersonLocal person ) throws javax.ejb.CreateException {
      // Set CMR fields
      setFlight(flight);
      setPerson(person);
   }

  
   /**
    * Return the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract ejb.reservations.entity.ReservationLightValue getReservationLightValue();

   /**
    * Set the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setReservationLightValue(ejb.reservations.entity.ReservationLightValue value);

   /**
    * Return the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract ejb.reservations.entity.ReservationValue getReservationValue();
   
   /**
    * Set the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setReservationValue(ejb.reservations.entity.ReservationValue value);

   /**
    * Create and return a value object populated with the data from
    * this bean.
    *
    * Standard method that must be on all Beans for the TreeBuilder to
    * work its magic.
    *
    * @return Returns a value object containing the data within this bean.
    *
    * @ejb.interface-method view-type="local"
    */
   public ejb.reservations.entity.ReservationValue getValueObject() {
      ejb.reservations.entity.ReservationValue valueObject = new ejb.reservations.entity.ReservationValue();
      valueObject.setReservationId(getReservationId());
      valueObject.setRegistrationUtc(getRegistrationUtc());
      valueObject.setComment(getComment());
      return valueObject;
   }

   /**
    * Creates an instance based on a value object
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param value the value object used to initialise the new instance
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public ejb.reservations.entity.ReservationPK ejbCreate(ejb.reservations.entity.ReservationLightValue value) throws javax.ejb.CreateException {
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      setReservationId(value.getReservationId());
      setRegistrationUtc(value.getRegistrationUtc());
      setComment(value.getComment());
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes this method immediately after it calls ejbCreate. 
    *
    * @param value the value object used to initialise the new instance
    */
   public void ejbPostCreate(ejb.reservations.entity.ReservationLightValue value) throws javax.ejb.CreateException {
   }
   // No H:\Projet\Middlegen\samples/src/middlegen/cmp20-reservations-class-code.txt found.

  public void ejbLoad() 
   {
   }

   public void ejbStore() 
   {
   }

   public void ejbActivate() 
   {
   }

   public void ejbPassivate() 
   {
   }

   public void setEntityContext(javax.ejb.EntityContext ctx) 
   {
   }

   public void unsetEntityContext() 
   {
   }

   public void ejbRemove() throws javax.ejb.RemoveException
   {

   }


}
