/*
 * Copyright (c) 2001, Aslak Hellesy, BEKK Consulting
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * - Neither the name of BEKK Consulting nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
package middlegen;

import junit.framework.*;

/**
 * JUnit test case
 *
 * @author <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @created 3. oktober 2001
 * @version $Id: DbNameConverterTest.java,v 1.1 2002/05/04 12:35:44 rinkrank Exp
 *      $
 */
public class DbNameConverterTest extends TestCase {

   /**
    * Creates new TestUtil
    *
    * @param name name of test case
    */
   public DbNameConverterTest(String name) {
      super(name);
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName1() {
      String convert = "FOOVER_BART_ZAPP";
      assertEquals(convert, "FooverBartZapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName2() {
      String convert = "foover_bart_zapp";
      assertEquals(convert, "FooverBartZapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName3() {
      String convert = "fooverbartzapp";
      assertEquals(convert, "Fooverbartzapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName4() {
      String convert = "fooverBartZapp";
      assertEquals(convert, "FooverBartZapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName5() {
      String convert = "FOOVER_bartZapp";
      assertEquals(convert, "FooverBartZapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }


   /**
    * A unit test for JUnit
    */
   public void testDbNameToVariableName6() {
      String convert = "fooverBart_ZAPP";
      assertEquals(convert, "FooverBartZapp", DbNameConverter.getInstance().tableNameToVariableName(convert));
   }
}

